package com.mycompany.commerce.foundation.server.services.rest.search.postprocessor.solr;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2013, 2014
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import com.ibm.commerce.foundation.common.util.logging.LoggingHelper;
import com.ibm.commerce.foundation.internal.server.services.search.config.solr.SolrSearchConfigurationRegistry;
import com.ibm.commerce.foundation.internal.server.services.search.metadata.solr.SolrSearchServiceConstants;
import com.ibm.commerce.foundation.server.services.dataaccess.SelectionCriteria;
import com.ibm.commerce.foundation.server.services.rest.search.SearchCriteria;
import com.ibm.commerce.foundation.server.services.search.SearchServiceConstants;
import com.ibm.commerce.foundation.server.services.search.query.SearchQueryPostprocessor;
import com.ibm.commerce.foundation.server.services.search.query.solr.AbstractSolrSearchQueryPostprocessor;
import com.ibm.commerce.foundation.server.services.valuemapping.ValueMappingService;

/**
 * This is a Solr specific implementation of the search query
 * post-processor for adding product's User Rating to the response. It expects
 * that product's were already added to the iSearchResponseObject by another
 * search.
 */
public class SolrRESTSearchCatalogEntryViewRatingQueryPostprocessor extends
		AbstractSolrSearchQueryPostprocessor implements
		SearchQueryPostprocessor {
	/**
	 * IBM Copyright notice field.
	 */
	private static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private static final String CLASSNAME = SolrRESTSearchCatalogEntryViewRatingQueryPostprocessor.class
			.getName();
	private static final Logger LOGGER = LoggingHelper
			.getLogger(SolrRESTSearchCatalogEntryViewRatingQueryPostprocessor.class);
	private String iComponentId = null;

	private SolrSearchConfigurationRegistry iSearchRegistry = null;

	private ValueMappingService iMappingService = null;
	private String iCatalogEntryViewMapper = null;
	private String iCatalogEntryViewName = null;
	private String iCatalogEntryViewUniqueIdName = null;
	
	
	private String iUserDataMapper = null;
	private String iUserDataName = null;
	private String iUserDataRatingName = null;
	
	/**
	 * Creates an instance of the search query post-processor for a particular
	 * component. This instance can be used and re-used for future search
	 * requests.
	 * 
	 * @param componentId
	 *            The target component.
	 */
	public SolrRESTSearchCatalogEntryViewRatingQueryPostprocessor(
			String componentId) {
		super();
		iComponentId = componentId;
		iSearchRegistry = SolrSearchConfigurationRegistry
				.getInstance(iComponentId);
	}

	/**
	 * @see com.ibm.commerce.foundation.server.services.search.query.solr.AbstractSolrSearchQueryPostprocessor#invoke(com.ibm.commerce.foundation.server.services.dataaccess.SelectionCriteria,
	 *      java.lang.Object[])
	 */
	public void invoke(SelectionCriteria selectionCriteria,
			Object... queryResponseObjects) throws RuntimeException {
		final String METHODNAME = "invoke(SelectionCriteria, Object[])";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] {
					selectionCriteria, queryResponseObjects });
		}

		super.invoke(selectionCriteria, queryResponseObjects);
		initMappingParameters();
		//Retrieve the mapped iSearchResponseObject object from the response
		List<Map<String, Object>> catalogEntryViews = (LinkedList<Map<String, Object>>) 
		iSearchResponseObject.getResponse().get("catalogEntryView"); 

		// 1. Compose the product ID list.
		if (catalogEntryViews != null) {
			List<String> productBeanIdList = new ArrayList<String>();
			for (Map<String, Object> catalogEntryView : catalogEntryViews) {
				String catalogEntryUniqueId = (String) catalogEntryView
						.get(iCatalogEntryViewUniqueIdName);
				if (catalogEntryUniqueId != null) {
					productBeanIdList.add(catalogEntryUniqueId);
				}
			}

			// 2. Fetch the info map of Product=>SKU List.
			if (!productBeanIdList.isEmpty()) {
				Map<String, Object> productRatingMap = populateProductUserData(productBeanIdList);

				// 3. Add the SKUs into the CatalogEntryView
				for (Map<String, Object> catalogEntryView : catalogEntryViews) {
					String catalogEntryUniqueId = (String) catalogEntryView
							.get(iCatalogEntryViewUniqueIdName);
					if (catalogEntryUniqueId != null) {
						if(catalogEntryView.containsKey(iUserDataName)){
							((HashMap<String,Object>) catalogEntryView.get(iUserDataName)).put(iUserDataRatingName, productRatingMap
									.get(catalogEntryUniqueId));
						}else{
							Map<String,Object> userData = new HashMap<String,Object>();
							userData.put(iUserDataRatingName,productRatingMap
									.get(catalogEntryUniqueId));
							catalogEntryView.put(iUserDataName, userData);
						}
					}
				}
			} else {
				if (LoggingHelper.isTraceEnabled(LOGGER)) {
					LOGGER.logp(LoggingHelper.DEFAULT_TRACE_LOG_LEVEL,
							CLASSNAME, METHODNAME, "property name: "
									+ "catalogEntryView"
									+ " cannot be found in the response.");
				}
			}
		}
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}

	/**
	 * Populating Product User data by reading it from an external source
	 * 
	 * @param productIdList
	 *            the product ID list.
	 * @return the map of Product=>UserData List.
	 */
	private Map<String, Object> populateProductUserData(
			List<String> catalogEntryIdList) {
		final String METHODNAME = "populateProductUserData";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, catalogEntryIdList);
		}
		Map<String, Object> productsUserDataMap = new HashMap<String, Object>();

		for (String catentry_id : catalogEntryIdList) {
			int rn = 0 + (int) (Math.random() * ((5) + 1));
			productsUserDataMap.put(catentry_id, rn);
		}

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME, productsUserDataMap);
		}
		return productsUserDataMap;
	}
	
	/**
	 * initMappingParameters for current processor 
	 */
	protected void initMappingParameters(){
		final String METHODNAME = "initMappingParameters";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] {});
		}			
		
		iMappingService = ValueMappingService.getInstance(iSelectionCriteria.getComponentId());
		SearchCriteria searchCriteria = new SearchCriteria(iSelectionCriteria);
		String resourceName = searchCriteria.getControlParameterValue(SearchServiceConstants.CTRL_PARAM_SEARCH_INTERNAL_SERVICE_RESOURCE);		
				
		//CatalogEntryView mapping
		iCatalogEntryViewMapper = getMapperName(searchCriteria, resourceName,
				SolrSearchServiceConstants.CATALOG_ENTRY_VIEW);		
		iCatalogEntryViewName = getExternalFieldName(iMappingService,iCatalogEntryViewMapper,SolrSearchServiceConstants.INTERNAL_CATALOG_ENTRY_VIEW_NAME);		
		iCatalogEntryViewUniqueIdName = getExternalFieldName(iMappingService,iCatalogEntryViewMapper,SolrSearchServiceConstants.CATENTRY_ID_SOLR_FIELD_NAME);
		
		iUserDataMapper = getMapperName(
				searchCriteria,
				resourceName,
				SolrSearchServiceConstants.CATALOG_ENTRY_VIEW
						+ SolrSearchServiceConstants.RESPONSE_TEMPLATE_VALUE_SEPARATOR
						+ SolrSearchServiceConstants.CATALOG_ENTRY_VIEW_USER_DATA_VALUES);
		
		iUserDataName = getExternalFieldName(iMappingService,iUserDataMapper,SolrSearchServiceConstants.CATALOG_ENTRY_VIEW_USER_DATA_VALUES);
		iUserDataRatingName = getExternalFieldName(iMappingService,iUserDataMapper,"rating");
		
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
	}
}
